/**

  Spago4Q - The Business Intelligence Free Platform

  Copyright (C) 2009 Engineering Ingegneria Informatica S.p.A.

  Spago4Q is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  Spago4Q is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

 **/
package it.eng.spago4q.extractors.msprojectclient;

import it.eng.spago4q.extractors.msprojectclient.bo.Baseline;
import it.eng.spago4q.extractors.msprojectclient.bo.Project;
import it.eng.spago4q.extractors.msprojectclient.bo.Task;

import java.util.List;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;
import java.util.Stack;

import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;

class SaxMSProjectClientUnmarshaller extends DefaultHandler {
	private String currentElementName = null;
	private String PROJECT = "Project";
	private String TASK = "Task";
	private String BASELINE = "Baseline";
	//
	private String TASKS = "Tasks";
	private String CALENDARS = "Calendars";
	private String EXTENDEDATTRIBUTES = "ExtendedAttributes";
	private String OUTLINECODE = "OutlineCode";
	private String TIMEPHASEDDATA = "TimephasedData";
	private String ASSIGNMENTS = "Assignments";
	private String RESOURCES = "Resources";

	private enum ElementType {
		PROJECT, TASKS, CALENDARS, TASK, BASELINE,
		EXTENDEDATTRIBUTES, OUTLINECODE, TIMEPHASEDDATA,
		ASSIGNMENTS, RESOURCES
	};

	private Stack<ElementType> elementTypeStack;
	private boolean isStackReadyForText;
	Set<String> elements;

	private Task currentTask = null;
	private Baseline currentBaseline = null;

	private List<Task> taskList = new ArrayList<Task>();
	private Project project = null;

	// -----

	public SaxMSProjectClientUnmarshaller() {
		elementTypeStack = new Stack<ElementType>();
		elements = new HashSet<String>();
		elements.addAll(Arrays.asList(Project.elements));
		elements.addAll(Arrays.asList(Task.elements));
		elements.addAll(Arrays.asList(Baseline.elements));
		isStackReadyForText = false;
	}

	public List<Task> getTaskList() {
		return taskList;
	}

	public Project getProject() {
		return project;
	}

	// ----- callbacks: -----

	// -----

	public void startElement(String uri, String localName, String qName,
			Attributes attribs) {

		if (localName != null && localName.equals(PROJECT)) {
//			System.out.println("--------------PROJECT--------------");
			elementTypeStack.push(ElementType.PROJECT);
			project = new Project();
		}
		if (localName != null && localName.equals(TASK)) {
//			System.out.println("--------------Task--------------");
			elementTypeStack.push(ElementType.TASK);
			currentTask = new Task();
		}
		if (localName != null && localName.equals(BASELINE)) {
//			System.out.println("--------------Baseline--------------");
			if (elementTypeStack.peek().equals(TASK)) {
				currentBaseline = new Baseline();
			}
			elementTypeStack.push(ElementType.BASELINE);

		}

		// Not considered
		if (localName != null && localName.equals(TASKS)) {
			elementTypeStack.push(ElementType.TASKS);
		}
		if (localName != null && localName.equals(CALENDARS)) {
			elementTypeStack.push(ElementType.CALENDARS);
		}
		if (localName != null && localName.equals(EXTENDEDATTRIBUTES)) {
			elementTypeStack.push(ElementType.EXTENDEDATTRIBUTES);
		}
		if (localName != null && localName.equals(OUTLINECODE)) {
			elementTypeStack.push(ElementType.OUTLINECODE);
		}
		if (localName != null && localName.equals(TIMEPHASEDDATA)) {
			elementTypeStack.push(ElementType.TIMEPHASEDDATA);
		}
		if (localName != null && localName.equals(ASSIGNMENTS)) {
			elementTypeStack.push(ElementType.ASSIGNMENTS);
		}

		if (localName != null && localName.equals(RESOURCES)) {
			elementTypeStack.push(ElementType.RESOURCES);
		}

		//
		if (localName != null && elements.contains(localName)) {
			isStackReadyForText = true;
			currentElementName = localName;
		} else {
			isStackReadyForText = false;
			currentElementName = null;
		}
	}

	public void endElement(String uri, String localName, String qName) {

		if (localName != null && localName.equals(PROJECT)) {
//			System.out.println("--------------END PROJECT--------------");
			elementTypeStack.pop();
		}
		if (localName != null && localName.equals(TASK)) {
//			System.out.println("--------------END Task--------------");
			elementTypeStack.pop();
			taskList.add(currentTask);
			currentTask = null;
		}
		if (localName != null && localName.equals(BASELINE)) {
//			System.out.println("--------------END Baseline--------------");
			elementTypeStack.pop();
			if (currentTask != null) {
				currentTask.addBaseline(currentBaseline);
			}
			currentBaseline = null;
		}

		// Not considered
		if (localName != null && localName.equals(TASKS)) {
			elementTypeStack.pop();
		}
		if (localName != null && localName.equals(CALENDARS)) {
			elementTypeStack.pop();
		}
		if (localName != null && localName.equals(EXTENDEDATTRIBUTES)) {
			elementTypeStack.pop();
		}
		if (localName != null && localName.equals(OUTLINECODE)) {
			elementTypeStack.pop();
		}
		if (localName != null && localName.equals(TIMEPHASEDDATA)) {
			elementTypeStack.pop();
		}
		if (localName != null && localName.equals(ASSIGNMENTS)) {
			elementTypeStack.pop();
		}
		if (localName != null && localName.equals(RESOURCES)) {
			elementTypeStack.pop();
		}
	}

	// -----

	public void characters(char[] data, int start, int length) {
		if (isStackReadyForText == true) {
			StringBuffer sb = new StringBuffer();
			switch (elementTypeStack.peek()) {
			case PROJECT:
				if (project != null && data != null
						&& project.getValue(currentElementName) == null) {
					sb.append(data, start, length);
					project.setValue(currentElementName, sb.toString());
				}
				break;
			case TASK:
				if (currentTask != null && data != null
						&& currentTask.getValue(currentElementName) == null) {
					sb.append(data, start, length);
					currentTask.setValue(currentElementName, sb.toString());
				}
				break;
			case BASELINE:
				if (currentTask != null && currentBaseline != null
						&& data != null
						&& currentBaseline.getValue(currentElementName) == null) {
					sb.append(data, start, length);
					currentBaseline.setValue(currentElementName, sb.toString());
				}
				break;
			default:
				break;
			}

		}
	}
}
